import { IRunnerOptions, ITaskFunction, ITasks, IStrategy, IDuration, IOnAdd, IOnStart, IOnRun, IOnRemove, IOnDone, IOnEnd } from '../Interface';
export declare class CoreRunner<T = any, TOptions = any> {
    static runnerCount: number;
    protected readonly options: IRunnerOptions<T, TOptions>;
    protected readonly strategy: IStrategy<T>;
    protected __working: boolean;
    protected __destroyed: boolean;
    protected __paused: boolean;
    tasks: ITasks<T>;
    duration: IDuration;
    constructor(strategy: IStrategy<T>, options?: Partial<IRunnerOptions<T, TOptions>>);
    isBusy(): boolean;
    protected setWorking(working: boolean): void;
    protected setDestroyed(destroyed: boolean): void;
    start(): boolean;
    destroy(): boolean;
    on(event: 'start', callback: IOnStart<T>): void;
    on(event: 'add', callback: IOnAdd<T>): void;
    on(event: 'remove', callback: IOnRemove<T>): void;
    on(event: 'run', callback: IOnRun<T>): void;
    on(event: 'done', callback: IOnDone<T>): void;
    on(event: 'end', callback: IOnEnd<T>): void;
    off(event: 'start' | 'add' | 'remove' | 'run' | 'done' | 'end'): void;
    add(task: ITaskFunction<T>, first?: boolean): number;
    addFirst(task: ITaskFunction<T>): number;
    addMultiple(tasks: ITaskFunction<T>[], first?: boolean): number;
    addMultipleFirst(tasks: ITaskFunction[]): number;
    remove(first?: boolean): ITaskFunction | undefined;
    removeFirst(): ITaskFunction | undefined;
    removeAt(index: number): ITaskFunction | undefined;
    removeRange(startIndex: number, endIndex: number): ITaskFunction[];
    removeAll(): ITaskFunction[];
}
